#include "StdAfx.h"
#include "hotkeyhandler.h"
#include "VolumeKey.h"

#define VOL_STEP (float)0.015

GUID g_guidMyContext = GUID_NULL;
HANDLE m_hMutex;
CRITICAL_SECTION g_CriticalSection;
CRITICAL_SECTION g_CriticalSection2;
CHotkeyHandler hk;
volatile bool g_bRun;

static IAudioEndpointVolume *g_pEndptVol = NULL;
static BOOL CALLBACK DlgProc(HWND, UINT, WPARAM, LPARAM);

#define EXIT_ON_ERROR(hr)  \
              if (FAILED(hr)) { goto Exit; }
#define ERROR_CANCEL(hr)  \
              if (FAILED(hr)) {  \
                  MessageBox(NULL, TEXT("The program will exit."),  \
                             TEXT("Fatal error"), MB_OK);  \
                  throw 0;}

void InitHotKey();
void StopHotKey();

void VolExit(void *);
void VolUp(void *);
void VolDown(void *);
void VolMute(void *);

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR lpCmdLine,
                     int nCmdShow)
{

	HANDLE hEvent = CreateEvent(NULL, FALSE, TRUE, L"VolumeKey_66be3f4c-9c87-46ee-b7c3-96db55eab9ca");
    if (NULL == hEvent)
       return FALSE;
 
    if (ERROR_ALREADY_EXISTS == ::GetLastError())
       return FALSE;

    HRESULT hr = S_OK;
    IMMDeviceEnumerator *pEnumerator = NULL;
    IMMDevice *pDevice = NULL;

    if (hPrevInstance)
    {
        return 0;
    }

    CoInitialize(NULL);

    hr = CoCreateGuid(&g_guidMyContext);
    EXIT_ON_ERROR(hr)

    // Get enumerator for audio endpoint devices.
    hr = CoCreateInstance(__uuidof(MMDeviceEnumerator),
                          NULL, CLSCTX_INPROC_SERVER,
                          __uuidof(IMMDeviceEnumerator),
                          (void**)&pEnumerator);
    EXIT_ON_ERROR(hr)

    // Get default audio-rendering device.
    hr = pEnumerator->GetDefaultAudioEndpoint(eRender, eConsole, &pDevice);
    EXIT_ON_ERROR(hr)

    hr = pDevice->Activate(__uuidof(IAudioEndpointVolume),
                           CLSCTX_ALL, NULL, (void**)&g_pEndptVol);
    EXIT_ON_ERROR(hr)

    EXIT_ON_ERROR(hr)

	InitHotKey();

	g_bRun=true;
	InitializeCriticalSection(&g_CriticalSection);
	InitializeCriticalSection(&g_CriticalSection2);
	while(true)
	{
		EnterCriticalSection(&g_CriticalSection);
		if(g_bRun==false)
		{
			break;
		} 
		LeaveCriticalSection(&g_CriticalSection);
		Sleep(1);
	}



Exit:
	LeaveCriticalSection(&g_CriticalSection);
	LeaveCriticalSection(&g_CriticalSection2);
	DeleteCriticalSection(&g_CriticalSection);
	DeleteCriticalSection(&g_CriticalSection2);
	StopHotKey();
	::ReleaseMutex( m_hMutex );
	::CloseHandle( m_hMutex );
	m_hMutex = NULL;
    if (FAILED(hr))
    {
        MessageBox(NULL, TEXT("This program requires Windows Vista."),
                   TEXT("Error termination"), MB_OK);
    }
    SAFE_RELEASE(pEnumerator)
    SAFE_RELEASE(pDevice)
    SAFE_RELEASE(g_pEndptVol)
    CoUninitialize();
    return 0;
}

void InitHotKey()
{
	int id;

	hk.RemoveHandler(id = 0);
	
	hk.InsertHandler(MOD_CONTROL, VK_ADD, VolUp, id);
	hk.InsertHandler(MOD_CONTROL, VK_SUBTRACT, VolDown, id);
	hk.InsertHandler(MOD_CONTROL, VK_MULTIPLY, VolMute, id);
	hk.InsertHandler(MOD_CONTROL|MOD_ALT, VK_DIVIDE, VolExit, id);
	hk.Start();
}

void StopHotKey()
{
	hk.Stop();
}

void VolExit(void *)
{
	EnterCriticalSection(&g_CriticalSection);
	g_bRun=false;
	LeaveCriticalSection(&g_CriticalSection);
}

void VolUp(void *)
{
	try
	{
		HRESULT hr;
		EnterCriticalSection(&g_CriticalSection2);
		hr = g_pEndptVol->SetMute(FALSE, &g_guidMyContext);
		ERROR_CANCEL(hr)
		float fVolume=0;
		hr = g_pEndptVol->GetMasterVolumeLevelScalar(&fVolume);
		ERROR_CANCEL(hr)
		fVolume+=VOL_STEP;
		if(fVolume>1)fVolume=1;
		hr = g_pEndptVol->SetMasterVolumeLevelScalar(fVolume, &g_guidMyContext);
		ERROR_CANCEL(hr)
		LeaveCriticalSection(&g_CriticalSection2);
	}
	catch(int nTmp)
	{
		LeaveCriticalSection(&g_CriticalSection2);
	}
}

void VolDown(void *)
{
	try
	{
		HRESULT hr;
		EnterCriticalSection(&g_CriticalSection2);
		hr = g_pEndptVol->SetMute(FALSE, &g_guidMyContext);
		ERROR_CANCEL(hr)
		float fVolume=0;
		hr = g_pEndptVol->GetMasterVolumeLevelScalar(&fVolume);
		ERROR_CANCEL(hr)
		fVolume-=VOL_STEP;
		if(fVolume<0)fVolume=0;
		hr = g_pEndptVol->SetMasterVolumeLevelScalar(fVolume, &g_guidMyContext);
		ERROR_CANCEL(hr)
		LeaveCriticalSection(&g_CriticalSection2);
	}
	catch(int nTmp)
	{
		LeaveCriticalSection(&g_CriticalSection2);
	}
}

void VolMute(void *)
{
	try
	{
		HRESULT hr;
		BOOL bMute=FALSE;
		EnterCriticalSection(&g_CriticalSection2);
		hr = g_pEndptVol->GetMute(&bMute);
		ERROR_CANCEL(hr)
		hr = g_pEndptVol->SetMute((bMute!=FALSE?FALSE:TRUE), &g_guidMyContext);
		ERROR_CANCEL(hr)
		LeaveCriticalSection(&g_CriticalSection2);
	}
	catch(int nTmp)
	{
		LeaveCriticalSection(&g_CriticalSection2);
	}
}